/*@
theory uses Base, Record, java.lang.String, java.io.PrintStream { 
  String: TYPE = java.lang.String.String;
  notFound: PREDICATE(Record.Array, Base.int, String) =
    PRED(a:Record.Array, n: Base.int, key: String): 
      FORALL(i:INT): 0 <= i AND i < n => a.value[i].key /= key;
}  
@*/
class Record
{
  String key; int value; 

  Record(String k, int v)  
  /*@ requires (VAR k).null = FALSE;
      assignable key, value; 
      ensures VAR key = VAR k AND VAR value = VAR v;
    @*/
  { key = k; value = v; }

  boolean equals(String k) 
  /*@ requires (VAR key).null = FALSE AND (VAR k).null = FALSE;
      ensures VALUE@NEXT = TRUE <=> (VAR key = VAR k); @*/
  { boolean e = key.equals(k); return e; }

  public static int search(Record[] a, String key) /*@
    requires 
      (VAR a).null = FALSE AND (VAR key).null = FALSE AND
      (FORALL(i:INT): 0 <= i AND i < VAR a.length =>
        (VAR a).value[i].null = FALSE AND 
        (VAR a).value[i].key.null = FALSE);
    ensures
      (LET result=VALUE@NEXT, n = (VAR a).length IN
        IF result = -1 THEN
          notFound(VAR a, n, VAR key)
        ELSE
          0 <= result AND result < n AND 
          notFound(VAR a, result, VAR key) AND 
          (VAR a).value[result].key = VAR key
        ENDIF);
  @*/
  {
    int n = a.length;
    for (int i=0; i<n; i++)
    /*@
      invariant (VAR a).null = FALSE AND (VAR key).null = FALSE
            AND (FORALL(i:INT): 0 <= i AND i < VAR a.length =>
                  (VAR a).value[i].null  = FALSE AND 
                  (VAR a).value[i].key.null = FALSE)
            AND VAR n = (VAR a).length
            AND 0 <= VAR i AND VAR i <= VAR n
            AND notFound(VAR a, VAR i, VAR key)
            AND (RETURNS@NEXT => 
                  VAR i < VAR n AND
                  (VAR a).value[VAR i].key = VAR key AND
                  VALUE@NEXT = VAR i);
      decreases VAR n - VAR i;
    @*/
    {
      Record r = new Record(a[i].key, a[i].value);
      boolean e = r.equals(key);
      if (e) return i;
    }
    return -1;
  }
  
  public static void main() /*@
    requires (VAR System.out).null = FALSE;
  @*/
  {
    int N = 10;
    Record[] a = new Record[N];
    for (int i=0; i<N; i++) 
    /*@ invariant 0 <= VAR i AND VAR i <= VAR N
              AND (VAR a).null = FALSE
              AND (OLD a).length = VAR N
              AND (VAR a).length = VAR N
              AND (FORALL(j:INT): 0 <= j AND j < VAR i =>
                     (VAR a).value[j].null = FALSE AND
                     (VAR a).value[j].key.null = FALSE);
        decreases VAR N - VAR i; @*/
      a[i] = new Record("abc", i);
    a[5] = new Record("xyz", 7);
    int i = search(a, "xyz");
    assert System.out != null;
    //@ assert (VAR System.out).null = FALSE;
    System.out.println(i);
  }
}







