// ----------------------------------------------------------------------------
// SourcePositionInterface.java
// A position in a source file.
// $Id: SourcePosition.java,v 1.7 2010/12/06 17:06:43 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for identifieric Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.InOut;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * A position in a source file.
 *
 */
public interface SourcePosition
{
  /***************************************************************************
   * Get the source file.
   * @return the source file (may be null, if there is none).
   **************************************************************************/
  public Source getSource();
  
  /***************************************************************************
   * Get the line number of the position.
   * @return the line number.
   **************************************************************************/
  public int getLine();
  
  /***************************************************************************
   * Get the column number of the position.
   * @return the column number.
   **************************************************************************/
  public int getColumn();
  
  /****************************************************************************
   * Get offset of denoted position
   * @return the number of characters up to (excluding) the position denoted
   * by the line and column index.
   ***************************************************************************/
  public int getOffset();
  
  /**************************************************************************
   * Get number of characters from current position to end of line.
   * @return the number of characters.
   *************************************************************************/
  public int getLength();
  
  /**************************************************************************
   * Get printable text representation of position.
   * @return the text representation.
   *************************************************************************/
  public String toString();
  
  /**************************************************************************
   * Get source line corresponding to position.
   * @return the source line of the position
   *************************************************************************/
  public String getSourceLine();
  
  /***************************************************************************
   * Register identifier with denoted position.
   * @param ident the identifier to be registered.
   **************************************************************************/
  public void setIdentifier(Identifier ident);
  
  /***************************************************************************
   * Get identifier registered with denoted position.
   * @return the registered identifier (may be null).
   **************************************************************************/
  public Identifier getIdentifier();
}

