// ----------------------------------------------------------------------------
// BrowserWindow.java
// A browser window.
// $Id: BrowserWindow.java,v 1.4 2011/06/22 21:23:40 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.SWT;

import fmrisc.ProgramExplorer.*;

import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.*;

/**
 * 
 * A browser window.
 *
 */
public final class BrowserWindow extends Composite
{
  // the GUI of the window
  private MainSWT gui;
  
  // the shell of the window
  private Shell shell;
  
  // the widgets
  private Browser browser = null; // the browser
  
  /****************************************************************************
   * Construct window connected to denoted GUI.
   * @param gui the GUI.
   * @param header the header text of the window.
   * @param width initial width
   * @param height initial height
   * @return a window connected to the GUI.
   ***************************************************************************/
  public static BrowserWindow construct(MainSWT gui, String header,
    int width, int height)
  {
    Display display = gui.getDisplay();
    Shell shell = new Shell(display, 
      SWT.CLOSE | SWT.MIN | SWT.MAX | SWT.RESIZE | SWT.TITLE | SWT.BORDER);
    shell.setText("RISC ProgramExplorer: " + header);
    Image riscImage = new Image(shell.getDisplay(),
        Main.class.getResourceAsStream("/" + MainSWT.riscImageName));
    shell.setImage(riscImage);
    return new BrowserWindow(gui, shell, width, height);
  }
  
  /****************************************************************************
   * Construct panel for denoted GUI and shell.
   * @param gui the GUI of the panel.
   * @param shell the shell to which the panel is attached.
   * @param width initial width
   * @param height initial height
   ***************************************************************************/
  public BrowserWindow(MainSWT gui, Shell shell, int width, int height)
  {
    // initialize window
    super(shell, SWT.NONE);
    this.gui = gui;
    this.shell = shell;
    
    // resize it
    shell.setSize(width, height);
    
    // set shell layout for shell and panel
    shell.setLayout(new FillLayout());
    setLayout(new FillLayout());
    
    // create widgets
    createMenu();
    createBrowser();
  }

  /****************************************************************************
   * Create browser.
   ***************************************************************************/
  private void createBrowser()
  {
    browser = new Browser(this, SWT.MOZILLA | SWT.BORDER);
    browser.setBackground(gui.getBackgroundColor());
  }
  
  /****************************************************************************
   * Get browser from window.
   * @return the browser.
   ***************************************************************************/
  public Browser getBrowser()
  {
    return browser;
  }
  
  /***************************************************************************
   * Create menu bar.
   **************************************************************************/
  private void createMenu()
  {
    new Menu(shell, SWT.BAR);
    shell.setMenuBar(null);
  }
}
