// ----------------------------------------------------------------------------
// BodySymbol.java
// The symbol for a variable introduced in a method/constructor body.
// $Id: BodySymbol.java,v 1.8 2010/03/03 14:01:19 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import java.io.PrintWriter;

import fmrisc.ProgramExplorer.InOut.SourcePosition;
import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Program.TypeExpression;
import fmrisc.ProgramExplorer.Syntax.Program.VariableDeclaration;

/**
 * 
 * The symbol for a variable introduced in a method/constructor body.
 *
 */
public final class BodySymbol extends LocalVariableSymbolBase
{    
  private final VariableDeclaration decl;
  
  /***************************************************************************
   * Construct body symbol from name and type.
   * @param name the name of the symbol.
   * @param type the type of the symbol.
   * @param decl the declaration introducing the variable.
   * @return the symbol (or null, if an error occurred)
   **************************************************************************/
  public static BodySymbol construct(Identifier name, TypeExpression type, 
    VariableDeclaration decl)
  {
    BodySymbol symbol = new BodySymbol(name, type, decl);
    return symbol;
  }
  
  /***************************************************************************
   * Construct body symbol from name and type.
   * @param name the name of the symbol.
   * @param type the type of the symbol.
   * @param decl the declaration introducing the variable.
   **************************************************************************/
  protected BodySymbol(Identifier name, TypeExpression type, 
    VariableDeclaration decl)
  {
    super(name, type);
    this.decl = decl;
  }

  /***************************************************************************
   * Get declaration of variable.
   * @return the declaration.
   **************************************************************************/
  public VariableDeclaration getDeclaration()
  {
    return decl;
  }
  
  /***************************************************************************
   * Print symbol on output stream.
   * @param out the output stream.
   **************************************************************************/
  public void print(PrintWriter out)
  {
    out.print("variable ");
    getType().print(out);
    out.print(" ");
    getName().print(out);
    Declaration decl = getDeclaration();
    if (decl == null) return;
    SourcePosition pos = decl.getPosition();
    out.print(" (");
    out.print(pos.toString());
    out.print(")");
  }
}


