// ----------------------------------------------------------------------------
// GlobalSymbolBase.java
// The base class of a symbol that can be globally referenced.
// $Id: GlobalSymbolBase.java,v 1.5 2010/10/05 15:53:03 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * The base class of a symbol that can be globally referenced.
 *
 */
public abstract class GlobalSymbolBase 
extends SymbolBase implements GlobalSymbol
{
  // the long name of the symbol
  private final Name lname;             
  
  /****************************************************************************
   * Construct a symbol that can be externally referenced.
   * @param name the short name of the symbol.
   * @param lname the long name of the symbol.
   ***************************************************************************/
  public GlobalSymbolBase(Identifier name, Name lname)
  {
    super(name);
    this.lname = lname;
  }
  
  /****************************************************************************
   * Construct a symbol that can be externally referenced.
   * @param container the container of the symbol.
   * @param name the name of the symbol.
   ***************************************************************************/
  public GlobalSymbolBase(GlobalSymbol container, Identifier name)
  {
    super(name);
    Name pname = container.getLongName();
    Identifier[] idents = pname.idents;
    if (idents.length == 1 && idents[0].getString().equals(""))
      lname = Name.construct(getName());
    else
      lname = Name.construct(pname, getName());
  }
  
  /****************************************************************************
   * Get the unique long name by which the symbol can be referenced.
   * @return the unique long name.
   ***************************************************************************/
  public Name getLongName()
  {
    return lname;
  }
}
