// ----------------------------------------------------------------------------
// GlobalVariableSymbol.java
// The base class of a variable symbol that can be externally referenced.
// $Id: GlobalVariableSymbolBase.java,v 1.6 2011/06/28 18:39:31 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Program.TypeExpression;

/**
 * 
 * The base class of a variable symbol that can be externally referenced.
 *
 */
public abstract class GlobalVariableSymbolBase 
extends VariableSymbolBase implements GlobalSymbol
{
  private final ClassSymbol csymbol; // the class holding the variable
  private final Name lname;          // the long name of the symbol
  
  /***************************************************************************
   * Construct global variable symbol.
   * @param csymbol the symbol of the class holding the variable.
   * @param type the type of the variable
   **************************************************************************/
  protected GlobalVariableSymbolBase(ClassSymbol csymbol, 
    Identifier name, TypeExpression type)
  {
    super(name, type);
    this.csymbol = csymbol;
    this.lname = Name.construct(csymbol.getLongName(), getName());
  }
  
  /****************************************************************************
   * Get the unique long name by which the symbol can be referenced.
   * @return the unique long name.
   ***************************************************************************/
  public Name getLongName()
  {
    return lname;
  }
  
  /****************************************************************************
   * Get the class to which the variable belongs.
   * @return the class of the variable
   ***************************************************************************/
  public ClassSymbol getClassSymbol()
  {
    return csymbol;
  }
}

