// ----------------------------------------------------------------------------
// Symbol.java
// The base class of symbols.
// $Id: SymbolBase.java,v 1.8 2009/05/13 16:10:58 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * The base class of symbols.
 *
 */
public abstract class SymbolBase implements Symbol
{
  private final Identifier name;
  
  /***************************************************************************
   * Construct symbol from name.
   * @param name the name of the symbol.
   **************************************************************************/
  protected SymbolBase(Identifier name)
  {
    this.name = name;

    // link identifier to symbol
    name.setSymbol(this);
  }
  
  /***************************************************************************
   * Get name associated to symbol.
   * @return the long name of the symbol.
   **************************************************************************/
  public Identifier getName()
  {
    return name;
  }
  
  /***************************************************************************
   * Get declaration of variable (if any).
   * @return the declaration (may be null).
   **************************************************************************/
  public Declaration getDeclaration()
  {
    return null;
  }
}


