// ----------------------------------------------------------------------------
// TypeCloner.java
// Clone a type expression transforming references to certain symbols.
// $Id: TypeCloner.java,v 1.3 2011/09/30 16:02:16 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Semantics;

import java.util.*;

import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Syntax.Logic.*;

/**
 * 
 * Clone a type expression transforming references to certain symbols.
 *
 */
public class TypeCloner extends ReferenceCloner
{
  /****************************************************************************
   * Create a clone (deep copy) of a tree replacing
   * references to certain symbols.
   * @param tree the tree to be cloned 
   * @param map a map of (original) symbols to (new) symbols.
   * @return the clone of the tree 
   ***************************************************************************/
  public static AST clone(AST tree, Map<Symbol,Symbol> map)
  {
    TypeCloner cloner = new TypeCloner(map);
    return cloner.clone0(tree);
  }
  
  /****************************************************************************
   * Create a cloner for the denoted map.
   * @param map a map of (original) symbols to (new) symbols.
   ***************************************************************************/
  private TypeCloner(Map<Symbol,Symbol> map)
  {
    super(map);
  }
  
  /****************************************************************************
   * Replace reference to symbol by base type and new reference name.
   * @param ref the reference to be replaced.
   ***************************************************************************/
  public void visit(Reference ref) 
  { 
    Symbol symbol = ref.getName().getBase().getSymbol();
    if (!(symbol instanceof TypeSymbol)) 
    {
      super.visit(ref);
      return;
    }
    TypeSymbol symbol0 = (TypeSymbol)symbol;
    Type type = symbol0.getType();
    if (type == ref)
      super.visit(ref);
    else
      type.accept(this); 
  }
}

