// ----------------------------------------------------------------------------
// ASTVisitorBaseOld.java
// Base class of abstract syntax tree visitors (outdated version).
// $Id: ASTVisitorBaseOld.java,v 1.1 2011/09/30 15:47:34 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General protected License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General protected License for more details.
//
// You should have received a copy of the GNU General protected License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax;

import fmrisc.ProgramExplorer.*;
import fmrisc.ProgramExplorer.Syntax.Program.*;
import fmrisc.ProgramExplorer.Syntax.Logic.*;
import fmrisc.ProgramExplorer.Syntax.Spec.*;
import fmrisc.ProgramExplorer.Judgements.*;

/**
 * Base class of abstract syntax tree visitors (outdated version).
 */
public abstract class ASTVisitorBaseOld implements ASTVisitorOld
{  
  // --------------------------------------------------------------------------
  // Note: we implement the dispatch of visit() by sequences of "instanceof"
  // tests. Alternatively, we might speedup dispatches by proceeding as follows:
  // 1) for every AST type T, we hash T.class to a unique numerical id.
  // 2) for a given tree, we lookup the id of tree.getClass() and use
  //    switch(id) to dispatch the appropriate visit() method.
  // This strategy would be applicable in those cases where the dispatch
  // is performed on the dynamic (actual) type of the object (rather than
  // on any supertype or interface type).
  // --------------------------------------------------------------------------
  
  /****************************************************************************
   * Visit an abstract syntax tree.
   * @param tree the node of the abstract syntax tree that is visited.
   * @return true iff no visit of child nodes is required any more
   ***************************************************************************/
  public boolean visit(AST tree)
  { 
    // we proceed recursively according to the derivation from ASTBase
    if (!(tree instanceof ASTBase)) 
      Main.internalError("tree must be instance of ASTBase");
        
    // top-level classification according to base class
    if (tree instanceof ASTSourceBase)
      return visit((ASTSourceBase)tree);
    if (tree instanceof ASTProgramBase)
      return visit((ASTProgramBase)tree);
    if (tree instanceof ASTLogicBase)
      return visit((ASTLogicBase)tree);
    
    // concrete childs of ASTBase
    if (tree instanceof Name)
      return visit((Name)tree);
    if (tree instanceof StatJudgement)
      return visit((StatJudgement)tree);
    
    // unexpected error
    Main.internalError("unknown kind of tree");
    return false;
  }
  
  protected boolean visit(Name tree) { return false; }
  protected boolean visit(StatJudgement tree) { return false; }
  
  // --------------------------------------------------------------------------
  //
  // ASTSourceBase
  //
  // --------------------------------------------------------------------------
  
  protected boolean visit(ASTSourceBase tree)
  {
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.DeclarationBase)
      return visit((fmrisc.ProgramExplorer.Syntax.Program.DeclarationBase)tree);
    if (tree instanceof Handler)
      return visit((Handler)tree);
    if (tree instanceof Identifier)
      return visit((Identifier)tree);
    if (tree instanceof StatementBase)
      return visit((StatementBase)tree);
    Main.internalError("unknown kind of source");
    return false;
  }
  
  protected boolean visit(Handler tree) { return false; }
  protected boolean visit(Identifier tree) { return false; }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.DeclarationBase tree)
  {
    if (tree instanceof CompilationUnitBase) 
      return visit((CompilationUnitBase)tree);
    if (tree instanceof TopDeclarationBase) 
      return visit((TopDeclarationBase)tree);
    if (tree instanceof VariableDeclarationBase) 
      return visit((VariableDeclarationBase)tree);
    Main.internalError("unknown kind of declaration");
    return false;
  }

  protected boolean visit(CompilationUnitBase tree) 
  { 
    if (tree instanceof ClassDeclaration) 
      return visit((ClassDeclaration)tree);
    if (tree instanceof TheoryDeclaration) 
      return visit((TheoryDeclaration)tree);
    Main.internalError("unknown kind of compilation unit");
    return false; 
  }
  
  protected boolean visit(ClassDeclaration tree) { return false; }
  protected boolean visit(TheoryDeclaration tree) { return false; }
  
  protected boolean visit(TopDeclarationBase tree) 
  { 
    if (tree instanceof GlobalVariableDeclaration) 
      return visit((GlobalVariableDeclaration)tree);
    if (tree instanceof ParamDeclarationBase) 
      return visit((ParamDeclarationBase)tree);
    Main.internalError("unknown kind of top-level declaration");
    return false;
  }
  
  protected boolean visit(GlobalVariableDeclaration tree) 
  { 
    if (tree instanceof ClassVariableDeclaration) 
      return visit((ClassVariableDeclaration)tree);
    if (tree instanceof ObjectVariableDeclaration) 
      return visit((ObjectVariableDeclaration)tree);
    Main.internalError("unknown kind of global variable declaration");
    return false;
  }
  
  protected boolean visit(ClassVariableDeclaration tree) { return false; }
  protected boolean visit(ObjectVariableDeclaration tree) { return false; }
  
  protected boolean visit(ParamDeclarationBase tree) 
  { 
    if (tree instanceof ConstructorDeclaration) 
      return visit((ConstructorDeclaration)tree);
    if (tree instanceof MethodDeclaration) 
      return visit((MethodDeclaration)tree);
    Main.internalError("unknown kind of parameterized declaration");
    return false;
  }
  
  protected boolean visit(ConstructorDeclaration tree) { return false; }
  
  protected boolean visit(MethodDeclaration tree) 
  { 
    if (tree instanceof ClassMethodDeclaration) 
      return visit((ClassMethodDeclaration)tree);
    if (tree instanceof ObjectMethodDeclaration) 
      return visit((ObjectMethodDeclaration)tree);
    Main.internalError("unknown kind of method declaration");
    return false;
  }
  
  protected boolean visit(ClassMethodDeclaration tree) { return false; }
  protected boolean visit(ObjectMethodDeclaration tree) { return false; }
  
  protected boolean visit(VariableDeclarationBase tree) 
  { 
    if (tree instanceof Parameter) 
      return visit((Parameter)tree);
    if (tree instanceof VariableStatement) 
      return visit((VariableStatement)tree);
    Main.internalError("unknown kind of variable declaration");
    return false;
  }
  
  protected boolean visit(Parameter tree) { return false; }
  protected boolean visit(VariableStatement tree) { return false; }
  
  protected boolean visit(StatementBase tree) 
  { 
    if (tree instanceof AssertionStatement) 
      return visit((AssertionStatement)tree);
    if (tree instanceof AssignmentStatement) 
      return visit((AssignmentStatement)tree);
    if (tree instanceof BlockStatement) 
      return visit((BlockStatement)tree);
    if (tree instanceof BreakStatement) 
      return visit((BreakStatement)tree);
    if (tree instanceof CallStatement) 
      return visit((CallStatement)tree);
    if (tree instanceof ConditionalStatement) 
      return visit((ConditionalStatement)tree);
    if (tree instanceof ContinueStatement) 
      return visit((ContinueStatement)tree);
    if (tree instanceof EmptyStatement) 
      return visit((EmptyStatement)tree);
    if (tree instanceof LoopStatementBase) 
      return visit((LoopStatementBase)tree);
    if (tree instanceof ReturnStatement) 
      return visit((ReturnStatement)tree);
    if (tree instanceof TryCatchStatement) 
      return visit((TryCatchStatement)tree);
    Main.internalError("unknown kind of statement");
    return false;
  }
  
  protected boolean visit(AssertionStatement tree) { return false; }
  protected boolean visit(AssignmentStatement tree) { return false; }
  protected boolean visit(BlockStatement tree) { return false; }
  protected boolean visit(BreakStatement tree) { return false; }
  protected boolean visit(ConditionalStatement tree) { return false; }
  protected boolean visit(ContinueStatement tree) { return false; }
  protected boolean visit(EmptyStatement tree) { return false; }
  protected boolean visit(ReturnStatement tree) { return false; }
  protected boolean visit(TryCatchStatement tree) { return false; }
  
  protected boolean visit(LoopStatementBase tree) 
  { 
    if (tree instanceof WhileLoopStatement) 
      return visit((WhileLoopStatement)tree);
    if (tree instanceof ForLoopStatement) 
      return visit((ForLoopStatement)tree);
    Main.internalError("unknown kind of looop");
    return false;
  }
  
  protected boolean visit(ForLoopStatement tree) { return false; }
  protected boolean visit(WhileLoopStatement tree) { return false; }
  
  protected boolean visit(CallStatement tree) 
  { 
    if (tree instanceof AssignCallStatement) 
      return visit((AssignCallStatement)tree);
    if (tree instanceof DeclCallStatement) 
      return visit((DeclCallStatement)tree);
    if (tree instanceof ThrowStatement) 
      return visit((ThrowStatement)tree);
    if (tree instanceof VoidCallStatement) 
      return visit((VoidCallStatement)tree);
    Main.internalError("unknown kind of method call");
    return false;
  }
  
  protected boolean visit(AssignCallStatement tree) 
  { 
    if (tree instanceof AssignNewStatement) 
      return visit((AssignNewStatement)tree);
    return false; 
  }
  protected boolean visit(AssignNewStatement tree) { return false; }
  
  protected boolean visit(DeclCallStatement tree) 
  { 
    if (tree instanceof DeclNewStatement) 
      return visit((DeclNewStatement)tree);
    return false; 
  }
  protected boolean visit(DeclNewStatement tree)  { return false; }
  
  protected boolean visit(ThrowStatement tree) { return false; }
  
  protected boolean visit(VoidCallStatement tree) { return false; }  
  
  // --------------------------------------------------------------------------
  //
  // ASTProgramBase
  //
  // --------------------------------------------------------------------------

  protected boolean visit(ASTProgramBase tree)
  {
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.ArrayType) 
      return visit((fmrisc.ProgramExplorer.Syntax.Program.ArrayType)tree);
    if (tree instanceof ASTProgramAnnotatedBase) 
      return visit((ASTProgramAnnotatedBase)tree);
    if (tree instanceof ImportBase) 
      return visit((ImportBase)tree);
    if (tree instanceof LeftValue) 
      return visit((LeftValue)tree);
    if (tree instanceof NamedType) 
      return visit((NamedType)tree);
    if (tree instanceof NewType) 
      return visit((NewType)tree);
    if (tree instanceof PrimitiveType) 
      return visit((PrimitiveType)tree);
    if (tree instanceof ProgramAnnotationBase) 
      return visit((ProgramAnnotationBase)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.SelectorBase) 
      return visit((fmrisc.ProgramExplorer.Syntax.Program.SelectorBase)tree);
    if (tree instanceof ValueExpressionBase) 
      return visit((ValueExpressionBase)tree);
    Main.internalError("unknown kind of program construct");
    return false;
  }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.ArrayType tree) { return false; }
  protected boolean visit(ASTProgramAnnotatedBase tree) 
  { 
    Main.internalError("unknown kind of annotated construct"); 
    return false; 
  }
  protected boolean visit(LeftValue tree) { return false; }
  protected boolean visit(NamedType tree) { return false; }
  protected boolean visit(NewType tree) { return false; }
  
  protected boolean visit(ImportBase tree)
  {
    if (tree instanceof PackageImport) 
      return visit((PackageImport)tree);
    if (tree instanceof ClassImport) 
      return visit((ClassImport)tree);
    Main.internalError("unknown kind of import statement");
    return false;
  }
  
  protected boolean visit(PackageImport tree) { return false; }
  protected boolean visit(ClassImport tree) { return false; }
  
  protected boolean visit(PrimitiveType tree) 
  { 
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.BooleanType) 
      return visit((fmrisc.ProgramExplorer.Syntax.Program.BooleanType)tree);
    if (tree instanceof CharType) 
      return visit((CharType)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.IntType) 
      return visit((fmrisc.ProgramExplorer.Syntax.Program.IntType)tree);
    if (tree instanceof NullType) 
      return visit((NullType)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.StringType) 
      return visit((fmrisc.ProgramExplorer.Syntax.Program.StringType)tree);
    if (tree instanceof VoidType) 
      return visit((VoidType)tree);
    Main.internalError("unknown kind of primitive type");
    return false;
  }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.BooleanType tree) { return false; }
  protected boolean visit(CharType tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.IntType tree) { return false; }
  protected boolean visit(NullType tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.StringType tree) { return false; }
  protected boolean visit(VoidType tree) { return false; }
  protected boolean visit(ExceptionType tree) { return false; }
  
  protected boolean visit(ProgramAnnotationBase tree) 
  { 
    if (tree instanceof SpecificationBase) 
      return visit((SpecificationBase)tree);
    Main.internalError("unknown kind of program annotation");
    return false;
  }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.SelectorBase tree)
  {
    if (tree instanceof IdentifierSelector) 
      return visit((IdentifierSelector)tree);
    if (tree instanceof IndexSelector) 
      return visit((IndexSelector)tree);
    if (tree instanceof MessageSelector) 
      return visit((MessageSelector)tree);
    Main.internalError("unknown kind of program construct");
    return false;
  }
  
  protected boolean visit(IdentifierSelector tree) { return false; }
  protected boolean visit(IndexSelector tree) { return false; }
  protected boolean visit(MessageSelector tree) { return false; }
  
  protected boolean visit(SpecificationBase tree) 
  { 
    if (tree instanceof MethodSpec) 
      return visit((MethodSpec)tree);
    if (tree instanceof UnitSpec) 
      return visit((UnitSpec)tree);
    if (tree instanceof ClassSpec) 
      return visit((ClassSpec)tree);
    if (tree instanceof LoopSpec) 
      return visit((LoopSpec)tree);
    if (tree instanceof StatementSpec) 
      return visit((StatementSpec)tree);
    Main.internalError("unknown kind of specification");
    return false;
  }
  
  protected boolean visit(MethodSpec tree) { return false; }
  protected boolean visit(UnitSpec tree) { return false; }
  protected boolean visit(ClassSpec tree) { return false; }
  protected boolean visit(LoopSpec tree) { return false; }
  protected boolean visit(StatementSpec tree) { return false; }
  
  protected boolean visit(ValueExpressionBase tree) 
  { 
    if (tree instanceof BinaryValueExpression)
      return visit((BinaryValueExpression)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.BooleanLiteral)
      return visit((fmrisc.ProgramExplorer.Syntax.Program.BooleanLiteral)tree);
    if (tree instanceof CharLiteral)
      return visit((CharLiteral)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.IntLiteral)
      return visit((fmrisc.ProgramExplorer.Syntax.Program.IntLiteral)tree);
    if (tree instanceof NewArrayExpression)
      return visit((NewArrayExpression)tree);
    if (tree instanceof NullLiteral)
      return visit((NullLiteral)tree);
    if (tree instanceof SelectorExpression)
      return visit((SelectorExpression)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Program.StringLiteral)
      return visit((fmrisc.ProgramExplorer.Syntax.Program.StringLiteral)tree);
    if (tree instanceof UnaryValueExpression)
      return visit((UnaryValueExpression)tree);
    if (tree instanceof VariableValue)
      return visit((VariableValue)tree);
    Main.internalError("unknown kind of value expression");
    return false;
  }
  
  protected boolean visit(CharLiteral tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.BooleanLiteral tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.IntLiteral tree) { return false; }
  protected boolean visit(NewArrayExpression tree) { return false; }
  protected boolean visit(NullLiteral tree) { return false; }
  protected boolean visit(SelectorExpression tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Program.StringLiteral tree) { return false; }
  protected boolean visit(VariableValue tree) { return false; }

  protected boolean visit(BinaryValueExpression tree) 
  { 
    if (tree instanceof AndExpression)
      return visit((AndExpression)tree);
    if (tree instanceof DividesExpression)
      return visit((DividesExpression)tree);
    if (tree instanceof EqualsExpression)
      return visit((EqualsExpression)tree);
    if (tree instanceof GreaterEqualExpression)
      return visit((GreaterEqualExpression)tree);
    if (tree instanceof GreaterExpression)
      return visit((GreaterExpression)tree);
    if (tree instanceof LessEqualExpression)
      return visit((LessEqualExpression)tree);
    if (tree instanceof LessExpression)
      return visit((LessExpression)tree);
    if (tree instanceof MinusExpression)
      return visit((MinusExpression)tree); 
    if (tree instanceof NotEqualsExpression)
      return visit((NotEqualsExpression)tree);
    if (tree instanceof OrExpression)
      return visit((OrExpression)tree);
    if (tree instanceof PercentExpression)
      return visit((PercentExpression)tree);
    if (tree instanceof PlusExpression)
      return visit((PlusExpression)tree);
    if (tree instanceof TimesExpression)
      return visit((TimesExpression)tree);
    Main.internalError("unknown kind of binary value expression");
    return false;
  }
  
  protected boolean visit(AndExpression tree) { return false; }
  protected boolean visit(DividesExpression tree) { return false; }
  protected boolean visit(EqualsExpression tree) { return false; }
  protected boolean visit(GreaterEqualExpression tree) { return false; }
  protected boolean visit(GreaterExpression tree) { return false; }
  protected boolean visit(LessEqualExpression tree) { return false; }
  protected boolean visit(LessExpression tree) { return false; }
  protected boolean visit(MinusExpression tree) { return false; }
  protected boolean visit(NotEqualsExpression tree) { return false; }
  protected boolean visit(OrExpression tree) { return false; }
  protected boolean visit(PercentExpression tree) { return false; }
  protected boolean visit(PlusExpression tree) { return false; }
  protected boolean visit(TimesExpression tree) { return false; }
  
  protected boolean visit(UnaryValueExpression tree) 
  { 
    if (tree instanceof NegationExpression)
      return visit((NegationExpression)tree);
    if (tree instanceof NotExpression)
      return visit((NotExpression)tree);
    Main.internalError("unknown kind of unary value expression");
    return false;
  }
  
  protected boolean visit(NegationExpression tree) { return false; }
  protected boolean visit(NotExpression tree) { return false; }
  
  // --------------------------------------------------------------------------
  //
  // ASTLogicBase
  //
  // --------------------------------------------------------------------------
  
  protected boolean visit(ASTLogicBase tree)
  {
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.DeclarationBase)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.DeclarationBase)tree);
    if (tree instanceof ExpressionBase)
      return visit((ExpressionBase)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.SelectorBase)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.SelectorBase)tree);
    if (tree instanceof TypedIdentifier)
      return visit((TypedIdentifier)tree);
    if (tree instanceof ValuedIdentifier)
      return visit((ValuedIdentifier)tree);
    Main.internalError("unknown kind of logic entity");
    return false;
  }
  
  protected boolean visit(TypedIdentifier tree) { return false; }
  protected boolean visit(ValuedIdentifier tree) { return false; }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.DeclarationBase tree)
  {
    if (tree instanceof DefinitionBase)
      return visit((DefinitionBase)tree);
    if (tree instanceof TypeDeclaration)
      return visit((TypeDeclaration)tree);
    if (tree instanceof ValueDeclarationClass)
      return visit((ValueDeclarationClass)tree);
    Main.internalError("unknown kind of logic declaration");
    return false;
  }
  
  protected boolean visit(DefinitionBase tree)
  {
    if (tree instanceof FormulaDefinition)
      return visit((FormulaDefinition)tree);
    if (tree instanceof ValueDefinitionClass)
      return visit((ValueDefinitionClass)tree);
    Main.internalError("unknown kind of definition");
    return false;
  }
  
  protected boolean visit(ValueDefinitionClass tree) { return false; }
  
  protected boolean visit(FormulaDefinition tree)
  {
    if (tree instanceof AxiomDefinition)
      return visit((AxiomDefinition)tree);
    return false;
  }
  
  protected boolean visit(AxiomDefinition tree) { return false; }
  
  protected boolean visit(TypeDeclaration tree)
  {
    if (tree instanceof TypeDefinition)
      return visit((TypeDefinition)tree);
    return false;
  }
  
  protected boolean visit(TypeDefinition tree) { return false; }
  
  protected boolean visit(ValueDeclarationClass tree)
  {
    if (tree instanceof ValueDeclarationFormula)
      return visit((ValueDeclarationFormula)tree);
    if (tree instanceof ValueDeclarationInitialized)
      return visit((ValueDeclarationInitialized)tree);
    return false;
  }
  
  protected boolean visit(ValueDeclarationFormula tree) { return false; }
  protected boolean visit(ValueDeclarationInitialized tree) { return false; }
  
  protected boolean visit(ExpressionBase tree)
  {
    if (tree instanceof ApplicationExpression)
      return visit((ApplicationExpression)tree);
    if (tree instanceof BinaryExpression)
      return visit((BinaryExpression)tree);
    if (tree instanceof Constant)
      return visit((Constant)tree);
    if (tree instanceof FormulaBase)
      return visit((FormulaBase)tree);
    if (tree instanceof IfThenElseExpression)
      return visit((IfThenElseExpression)tree);
    if (tree instanceof LetExpression)
      return visit((LetExpression)tree);
    if (tree instanceof PostfixApplication)
      return visit((PostfixApplication)tree);
    if (tree instanceof QuantifiedExpression)
      return visit((QuantifiedExpression)tree);
    if (tree instanceof Reference)
      return visit((Reference)tree);
    if (tree instanceof TermBase)
      return visit((TermBase)tree);
    if (tree instanceof TypeBase)
      return visit((TypeBase)tree);
    if (tree instanceof UnaryExpression)
      return visit((UnaryExpression)tree);
    Main.internalError("unknown kind of logic expression");
    return false;
  }
  
  protected boolean visit(Reference tree) { return false; }
  
  protected boolean visit(ApplicationExpression tree)
  {
    if (tree instanceof TermApplication)
      return visit((TermApplication)tree);
    Main.internalError("unknown kind of application expression");
    return false;
  }
  
  protected boolean visit(TermApplication tree)
  {
    if (tree instanceof AtomicFormula)
      return visit((AtomicFormula)tree);
    if (tree instanceof FunctionApplication)
      return visit((FunctionApplication)tree);
    Main.internalError("unknown kind of term application");
    return false;
  }
  
  protected boolean visit(AtomicFormula tree) { return false; }
  protected boolean visit(FunctionApplication tree) { return false; }
  
  protected boolean visit(BinaryExpression tree)
  {
    if (tree instanceof BinaryAtomicFormula)
      return visit((BinaryAtomicFormula)tree);
    if (tree instanceof BinaryFormula)
      return visit((BinaryFormula)tree);
    if (tree instanceof BinaryTerm)
      return visit((BinaryTerm)tree);
    Main.internalError("unknown kind of binary expression");
    return false;
  }
  
  protected boolean visit(BinaryAtomicFormula tree)
  {
    if (tree instanceof EqualsFormula)
      return visit((EqualsFormula)tree);
    if (tree instanceof GreaterEqualFormula)
      return visit((GreaterEqualFormula)tree);
    if (tree instanceof GreaterFormula)
      return visit((GreaterFormula)tree);
    if (tree instanceof LessEqualFormula)
      return visit((LessEqualFormula)tree);
    if (tree instanceof LessFormula)
      return visit((LessFormula)tree);
    if (tree instanceof NotEqualsFormula)
      return visit((NotEqualsFormula)tree);
    if (tree instanceof SimilarFormula)
      return visit((SimilarFormula)tree);
    Main.internalError("unknown kind of binary atomic formula");
    return false;
  }
  
  protected boolean visit(EqualsFormula tree) { return false; }
  protected boolean visit(GreaterEqualFormula tree) { return false; }
  protected boolean visit(GreaterFormula tree) { return false; }
  protected boolean visit(LessEqualFormula tree) { return false; }
  protected boolean visit(LessFormula tree) { return false; }
  protected boolean visit(NotEqualsFormula tree) { return false; }
  protected boolean visit(SimilarFormula tree) { return false; }
  
  protected boolean visit(BinaryFormula tree)
  {
    if (tree instanceof AndFormula)
      return visit((AndFormula)tree);
    if (tree instanceof EquivalentFormula)
      return visit((EquivalentFormula)tree);
    if (tree instanceof ImpliesFormula)
      return visit((ImpliesFormula)tree);
    if (tree instanceof NotEquivalentFormula)
      return visit((NotEquivalentFormula)tree);
    if (tree instanceof OrFormula)
      return visit((OrFormula)tree);
    Main.internalError("unknown kind of binary formula");
    return false;
  }
  
  protected boolean visit(AndFormula tree) { return false; }
  protected boolean visit(EquivalentFormula tree) { return false; }
  protected boolean visit(ImpliesFormula tree) { return false; }
  protected boolean visit(NotEquivalentFormula tree) { return false; }
  protected boolean visit(OrFormula tree) { return false; }
  
  protected boolean visit(BinaryTerm tree)
  {
    if (tree instanceof DividesTerm)
      return visit((DividesTerm)tree);
    if (tree instanceof MinusTerm)
      return visit((MinusTerm)tree);
    if (tree instanceof PlusTerm)
      return visit((PlusTerm)tree);
    if (tree instanceof TimesTerm)
      return visit((TimesTerm)tree);
    Main.internalError("unknown kind of binary term");
    return false;
  }
  
  protected boolean visit(DividesTerm tree) { return false; }
  protected boolean visit(MinusTerm tree) { return false; }
  protected boolean visit(PlusTerm tree) { return false; }
  protected boolean visit(TimesTerm tree) { return false; }
  
  protected boolean visit(Constant tree)
  {
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.BooleanLiteral)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.BooleanLiteral)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.IntLiteral)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.IntLiteral)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.StringLiteral)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.StringLiteral)tree);
    if (tree instanceof StateLiteral)
      return visit((StateLiteral)tree);
    Main.internalError("unknown kind of constant");
    return false;
  }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.IntLiteral tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.StringLiteral tree) { return false; }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.BooleanLiteral tree) 
  {
    if (tree instanceof True)
      return visit((True)tree);
    if (tree instanceof False)
      return visit((False)tree);
    Main.internalError("unknown kind of boolean literal");
    return false;
  }
  
  protected boolean visit(True tree) { return false; }
  protected boolean visit(False tree) { return false; }
  
  protected boolean visit(StateLiteral tree) 
  {
    if (tree instanceof OldState)
      return visit((OldState)tree);
    if (tree instanceof NewState)
      return visit((NewState)tree);
    Main.internalError("unknown kind of state literal");
    return false;
  }
  
  protected boolean visit(OldState tree) { return false; }
  protected boolean visit(NewState tree) { return false; }
  
  protected boolean visit(FormulaBase tree)
  {
    if (tree instanceof WritesOnlyFormula)
      return visit((WritesOnlyFormula)tree);
    Main.internalError("unknown kind of formula");
    return false;
  }
  
  protected boolean visit(WritesOnlyFormula tree)
  {
    if (tree instanceof ReadsOnlyFormula)
      return visit((ReadsOnlyFormula)tree);
    return false;
  }
  
  protected boolean visit(ReadsOnlyFormula tree) { return false; }
  
  protected boolean visit(IfThenElseExpression tree)
  {
    if (tree instanceof IfThenElseFormula)
      return visit((IfThenElseFormula)tree);
    if (tree instanceof IfThenElseTerm)
      return visit((IfThenElseTerm)tree);
    Main.internalError("unknown kind of conditional expression");
    return false;
  }
  
  protected boolean visit(IfThenElseFormula tree) { return false; }
  protected boolean visit(IfThenElseTerm tree) { return false; }
  
  protected boolean visit(LetExpression tree)
  {
    if (tree instanceof LetFormula)
      return visit((LetFormula)tree);
    if (tree instanceof LetTerm)
      return visit((LetTerm)tree);
    Main.internalError("unknown kind of let expression");
    return false;
  }
  
  protected boolean visit(LetFormula tree) { return false; }
  protected boolean visit(LetTerm tree) { return false; }
  
  protected boolean visit(PostfixApplication tree)
  {
    if (tree instanceof PostfixFormula)
      return visit((PostfixFormula)tree);
    if (tree instanceof PostfixTerm)
      return visit((PostfixTerm)tree);
    Main.internalError("unknown kind of postfix application");
    return false;
  }
  
  protected boolean visit(PostfixFormula tree)
  {
    if (tree instanceof StateBreaks)
      return visit((StateBreaks)tree);
    if (tree instanceof StateContinues)
      return visit((StateContinues)tree);
    if (tree instanceof StateExecutes)
      return visit((StateExecutes)tree);
    if (tree instanceof StateReturns)
      return visit((StateReturns)tree);
    if (tree instanceof StateThrows)
      return visit((StateThrows)tree);
    if (tree instanceof StateThrowsException)
      return visit((StateThrowsException)tree);
    Main.internalError("unknown kind of postfix formula");
    return false;
  }
  
  protected boolean visit(StateBreaks tree) { return false; }
  protected boolean visit(StateContinues tree) { return false; }
  protected boolean visit(StateExecutes tree) { return false; }
  protected boolean visit(StateReturns tree) { return false; }
  protected boolean visit(StateThrows tree) { return false; }
  protected boolean visit(StateThrowsException tree) { return false; }
  
  protected boolean visit(PostfixTerm tree)
  {
    if (tree instanceof StateValue)
      return visit((StateValue)tree);
    if (tree instanceof StateMessage)
      return visit((StateMessage)tree);
    Main.internalError("unknown kind of postfix term");
    return false;
  }
  
  protected boolean visit(StateValue tree) { return false; }
  protected boolean visit(StateMessage tree) { return false; }
  
  protected boolean visit(QuantifiedExpression tree)
  {
    if (tree instanceof QuantifiedFormula)
      return visit((QuantifiedFormula)tree);
    if (tree instanceof QuantifiedTerm)
      return visit((QuantifiedTerm)tree);
    Main.internalError("unknown kind of quantified expression");
    return false;
  }
  
  protected boolean visit(QuantifiedFormula tree)
  {
    if (tree instanceof ForallFormula)
      return visit((ForallFormula)tree);
    if (tree instanceof ExistsFormula)
      return visit((ExistsFormula)tree);
    Main.internalError("unknown kind of quantified formula");
    return false;
  }
  
  protected boolean visit(ForallFormula tree) { return false; }
  protected boolean visit(ExistsFormula tree) { return false; }
  
  protected boolean visit(QuantifiedTerm tree)
  {
    if (tree instanceof ArrayTerm)
      return visit((ArrayTerm)tree);
    if (tree instanceof LambdaTerm)
      return visit((LambdaTerm)tree);
    if (tree instanceof LambdaFormula)
      return visit((LambdaFormula)tree);
    Main.internalError("unknown kind of quantified term");
    return false;
  }
  
  protected boolean visit(ArrayTerm tree) { return false; }
  protected boolean visit(LambdaTerm tree) { return false; }
  protected boolean visit(LambdaFormula tree) { return false; }
  
  protected boolean visit(TermBase tree)
  {
    if (tree instanceof ProgramVariableBase)
      return visit((ProgramVariableBase)tree);
    if (tree instanceof RecordTerm)
      return visit((RecordTerm)tree);
    if (tree instanceof SelectionTerm)
      return visit((SelectionTerm)tree);
    if (tree instanceof TupleTerm)
      return visit((TupleTerm)tree);
    if (tree instanceof UpdateTerm)
      return visit((UpdateTerm)tree);
    Main.internalError("unknown kind of term");
    return false;
  }
  
  protected boolean visit(RecordTerm tree) { return false; }
  protected boolean visit(SelectionTerm tree) { return false; }
  protected boolean visit(TupleTerm tree) { return false; }
  protected boolean visit(UpdateTerm tree) { return false; }
  
  protected boolean visit(ProgramVariableBase tree)
  {
    if (tree instanceof PostVariable)
      return visit((PostVariable)tree);
    if (tree instanceof PreVariable)
      return visit((PreVariable)tree);
    Main.internalError("unknown kind of program variable");
    return false;
  }
  
  protected boolean visit(PostVariable tree) { return false; }
  protected boolean visit(PreVariable tree) { return false; }
  
  protected boolean visit(TypeBase tree)
  {
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.ArrayType)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.ArrayType)tree);
    if (tree instanceof AtomicType)
      return visit((AtomicType)tree);
    if (tree instanceof FunctionType)
      return visit((FunctionType)tree);
    if (tree instanceof RecordType)
      return visit((RecordType)tree);
    if (tree instanceof StateType)
      return visit((StateType)tree);
    if (tree instanceof SubrangeType)
      return visit((SubrangeType)tree);
    if (tree instanceof SubType)
      return visit((SubType)tree);
    if (tree instanceof TupleType)
      return visit((TupleType)tree);
    Main.internalError("unknown kind of type");
    return false;
  }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.ArrayType tree) { return false; }
  protected boolean visit(FunctionType tree) { return false; }
  protected boolean visit(RecordType tree) { return false; }
  protected boolean visit(StateType tree) { return false; }
  protected boolean visit(SubrangeType tree) { return false; }
  protected boolean visit(SubType tree) { return false; }
  protected boolean visit(TupleType tree) { return false; }
  
  protected boolean visit(AtomicType tree) 
  {
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.BooleanType)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.BooleanType)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.IntType)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.IntType)tree);
    if (tree instanceof NatType)
      return visit((NatType)tree);
    if (tree instanceof RealType)
      return visit((RealType)tree);
    if (tree instanceof Real0Type)
      return visit((Real0Type)tree);
    if (tree instanceof fmrisc.ProgramExplorer.Syntax.Logic.StringType)
      return visit((fmrisc.ProgramExplorer.Syntax.Logic.StringType)tree);
    // type may be user-defined
    // Main.internalError("unknown kind of type");
    return false;
  }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.BooleanType tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.IntType tree) { return false; }
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.StringType tree) { return false; }
  protected boolean visit(NatType tree) { return false; }
  protected boolean visit(RealType tree) { return false; }
  protected boolean visit(Real0Type tree) { return false; }
  
  protected boolean visit(UnaryExpression tree)
  {
    if (tree instanceof UnaryFormula)
      return visit((UnaryFormula)tree);
    if (tree instanceof UnaryTerm)
      return visit((UnaryTerm)tree);
    Main.internalError("unknown kind of unary expression");
    return false;
  }
  
  protected boolean visit(UnaryFormula tree)
  {
    if (tree instanceof NotFormula)
      return visit((NotFormula)tree);
    Main.internalError("unknown kind of unary formula");
    return false;
  }
  
  protected boolean visit(NotFormula tree) { return false; }
  
  protected boolean visit(UnaryTerm tree)
  {
    if (tree instanceof NegationTerm)
      return visit((NegationTerm)tree);
    Main.internalError("unknown kind of unary term");
    return false;
  }
  
  protected boolean visit(NegationTerm tree) { return false; }
  
  protected boolean visit(fmrisc.ProgramExplorer.Syntax.Logic.SelectorBase tree)
  {
    if (tree instanceof SelectorIdentifier)
      return visit((SelectorIdentifier)tree);
    if (tree instanceof SelectorIndex)
      return visit((SelectorIndex)tree);
    if (tree instanceof SelectorNumber)
      return visit((SelectorNumber)tree);
    Main.internalError("unknown kind of selector");
    return false;
  }
  
  protected boolean visit(SelectorIdentifier tree) { return false; }
  protected boolean visit(SelectorIndex tree) { return false; }
  protected boolean visit(SelectorNumber tree) { return false; }
}
