// ----------------------------------------------------------------------------
// ApplicationExpression.java
// An application of one expression to some other expression(s).
// $Id: ApplicationExpression.java,v 1.9 2011/09/30 15:47:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * An application of one expression to some other expression(s).
 *
 */
public abstract class ApplicationExpression extends ExpressionBase
{
  protected final Expression fun;
  protected final Expression[] args;
  
  /***************************************************************************
   * Construct application expression.
   * @param fun the expression that is being applied.
   * @param args the arguments to which the expression is applied.
   **************************************************************************/
  protected ApplicationExpression(Expression fun, Expression[] args)
  {
    this.fun = fun;
    this.args = args;
  }
  
  /****************************************************************************
   * Get the expression that is applied.
   * @return the expression that is applied.
   ***************************************************************************/
  public Expression getFunctionExpression()
  {
    return fun;
  }
  
  /****************************************************************************
   * Get the argument expressions.
   * @return the argument expressions.
   ***************************************************************************/
  public Expression[] getArgumentExpressions()
  {
    return args;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    super.accept(visitor);
    fun.accept(visitor);
    accept(args, visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);;
    fun.accept(visitor);
    accept(args, visitor);
  }
}

