// ----------------------------------------------------------------------------
// ArrayTerm.java
// An array term.
// $Id: ArrayTerm.java,v 1.5 2011/09/30 15:47:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * An array term.
 *
 */
public final class ArrayTerm extends QuantifiedTerm
{
  /***************************************************************************
   * Construct a function expression.
   * @param vars the bound variables of the term.
   * @param body the body of the term.
   * @return the term.
   **************************************************************************/
  public static ArrayTerm construct(TypedIdentifier[] vars, Term body)
  {
    return new ArrayTerm(vars, body);
  }
  
  /***************************************************************************
   * Construct a function expression.
   * @param vars the bound variables of the term.
   * @param body the body of the term.
   **************************************************************************/
  private ArrayTerm(TypedIdentifier[] vars, Term body)
  {
    super(vars, body);
  }
  
  /***************************************************************************
   * Get the body of the term.
   * @return the body of the term.
   **************************************************************************/
  public Term getBody()
  {
    return (Term)body;
  }
  
  /***************************************************************************
   * Get string representation of quantifier.
   * @return string representation of the quantifier.
   **************************************************************************/
  public String getQuantifier()
  {
    return "ARRAY";
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}