// ----------------------------------------------------------------------------
// BinaryFormula.java
// Base class of binary formulas.
// $Id: BinaryFormula.java,v 1.4 2011/09/30 15:47:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the formulas of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * Base class of binary formulas.
 *
 */
public abstract class BinaryFormula extends BinaryExpression implements Formula
{
  /****************************************************************************
   * Construct binary formula.
   * @param first the first formula.
   * @param second the second formula.
   ***************************************************************************/
  protected BinaryFormula(Formula first, Formula second)
  {
    super(first, second);
  }
  
  /***************************************************************************
   * Get first subformula.
   * @return the first subformula.
   **************************************************************************/
  public Formula getFirst()
  {
    return (Formula)first;
  }
  
  /***************************************************************************
   * Get second subformula.
   * @return the second subformula.
   **************************************************************************/
  public Formula getSecond()
  {
    return (Formula)second;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    super.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}

