// ----------------------------------------------------------------------------
// BitFalse.java
// A literal denoting the bit value "0".
// $Id: BitFalse.java,v 1.1 2012/10/18 16:12:57 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Main;
import fmrisc.ProgramExplorer.Syntax.ASTVisitor;

/**
 * 
 * A literal denoting the bit value "0".
 *
 */
public final class BitFalse extends BitLiteral
{  
  // same instance is returned
  private static final BitFalse value = new BitFalse();
  
  /****************************************************************************
   * Construct literal.
   * @return the new literal.
   ***************************************************************************/
  public static BitFalse construct()
  {
    return value;
  }
  
  /****************************************************************************
   * Construct literal.
   ***************************************************************************/
  private BitFalse()
  {
    super("BITFALSE");
  }
  
  /***************************************************************************
   * Get name of constant.
   * @return the name.
   **************************************************************************/
  public String getName()
  {
    if (Main.toLogical())
      return "FALSE";
    else
      return super.getName();
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
