// ----------------------------------------------------------------------------
// LetFormula.java
// A formula with local value definitions.
// $Id: LetFormula.java,v 1.4 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the formulas of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.ASTVisitor;

/**
 * 
 * A formula with local value definitions.
 *
 */
public final class LetFormula extends LetExpression implements Formula
{ 
  /***************************************************************************
   * Construct formula with local value definitions.
   * @param d the definitions.
   * @param b the body of the formula.
   * @return the formula
   **************************************************************************/
  public static LetFormula construct(ValueDefinition[] d, Formula b)
  {
    return new LetFormula(d, b);
  }
  
  /***************************************************************************
   * Construct formula with local bindings.
   * @param d the declarations.
   * @param b the body of the formula.
   **************************************************************************/
  private LetFormula(ValueDefinition[] d, Formula b)
  {
    super(d, b);
  }
  
  /***************************************************************************
   * Get definitions.
   * @return the definitions.
   **************************************************************************/
  public ValueDefinition[] getDefinitions()
  {
    return (ValueDefinition[])definitions;
  }
  
  /***************************************************************************
   * Get body.
   * @return the body.
   **************************************************************************/
  public Formula getBody()
  {
    return (Formula)body;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
