// ----------------------------------------------------------------------------
// PostfixApplication.java
// A postfix application of one expression to some other expression.
// $Id: PostfixApplication.java,v 1.7 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * A postfix application of one expression to some other expression.
 *
 */
public abstract class PostfixApplication extends ExpressionBase
{
  public final static String operator = "@";
  
  protected final Expression arg;
  protected final Identifier fun;
  
  /***************************************************************************
   * Construct postfix application expression.
   * @param arg the argument to which the function is applied.
   * @param fun the function that is being applied.
   **************************************************************************/
  protected PostfixApplication(Expression arg, Identifier fun)
  {
    this.arg = arg;
    this.fun = fun;
  }
  
  /***************************************************************************
   * Get the argument.
   * @return the argument.
   **************************************************************************/
  public Expression getArgument()
  {
    return arg;
  }
  
  /***************************************************************************
   * Get the function.
   * @return the function.
   **************************************************************************/
  public Identifier getIdentifier()
  {
    return fun;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    arg.accept(visitor);
    fun.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    arg.accept(visitor);
    fun.accept(visitor);
  }
}


