// ----------------------------------------------------------------------------
// ProgramVariable.java
// Base class of a reference to a program variable.
// $Id: ProgramVariableBase.java,v 1.6 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.ASTVisitorOld;
import fmrisc.ProgramExplorer.Syntax.ASTVisitor;
import fmrisc.ProgramExplorer.Syntax.Name;

/**
 * 
 * Base class of a reference to a program variable.
 *
 */
public abstract class ProgramVariableBase extends TermBase 
  implements ProgramVariable
{
  public final static int precedence  = 148; 
  protected final Name name;
  
  /***************************************************************************
   * Construct a program variable.
   * @param name the name of the variable.
   **************************************************************************/
  protected ProgramVariableBase(Name name)
  {
    this.name = name;
  }
  
  /***************************************************************************
   * Get name of variable.
   * @return the name.
   **************************************************************************/
  public Name getName()
  {
    return name;
  }
  
  /***************************************************************************
   * Get string representation of keyword.
   * @return string representation of the keyword.
   **************************************************************************/
  public abstract String getKeyword();
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    name.accept(visitor);
  }

  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    name.accept(visitor);
  }
  
  /***************************************************************************
   * Get precedence value of the term operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
}
