// ----------------------------------------------------------------------------
// QuantifiedTerm.java
// Base class of quantified terms.
// $Id: QuantifiedTerm.java,v 1.7 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * Base class of quantified terms.
 *
 */
public abstract class QuantifiedTerm extends QuantifiedExpression 
  implements Term
{
  public final static int precedence  = 10; 
  
  private Type type = null; // the type annotation
  
  /****************************************************************************
   * Set type annotation of term.
   * @param type the type annotation.
   ***************************************************************************/
  public void setType(Type type)
  {
    this.type = type;
  }
  
  /****************************************************************************
   * Set type annotation of term.
   * @return the type annotation.
   ***************************************************************************/
  public Type getType()
  {
    return type;
  }
  
  /***************************************************************************
   * Construct a quantified term.
   * @param vars the bound variables of the term.
   * @param body the body of the term.
   **************************************************************************/
  protected QuantifiedTerm(TypedIdentifier[] vars, Expression body)
  {
    super(vars, body);
  }
  
  /***************************************************************************
   * Get precedence value of the expression operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
