// ----------------------------------------------------------------------------
// Reference.java
// A reference to a logical variable (or constant).
// $Id: Reference.java,v 1.12 2017/07/04 08:04:38 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * A reference to a logical variable (or constant).
 *
 */
public final class Reference extends ExpressionBase 
  implements Term, Formula, Type
{ 
  private final Name name;
  
  private Type type = null; // the type annotation
  
  /****************************************************************************
   * Set type annotation of term.
   * @param type the type annotation.
   ***************************************************************************/
  public void setType(Type type)
  {
    this.type = type;
  }
  
  /****************************************************************************
   * Get type annotation of term.
   * @return the type annotation.
   ***************************************************************************/
  public Type getType()
  {
    return type;
  }
  
  /***************************************************************************
   * Construct a reference to a logical variable (or constant).
   * @param name the name of the variable.
   * @return the reference.
   **************************************************************************/
  public static Reference construct(Name name)
  {
    return new Reference(name);
  }
  
  /***************************************************************************
   * Construct a reference to a logical variable (or constant).
   * @param ident the name of the variable.
   * @return the reference.
   **************************************************************************/
  public static Reference construct(Identifier ident)
  {
    return new Reference(Name.construct(ident));
  }
  
  /***************************************************************************
   * Construct a reference to a logical variable (or constant).
   * @param name the name of the variable.
   **************************************************************************/
  private Reference(Name name)
  {
    this.name = name;
  }
  
  /***************************************************************************
   * Get name of the variable.
   * @return the name.
   **************************************************************************/
  public Name getName()
  {
    return name;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    name.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    name.accept(visitor);
  }
}
