// ----------------------------------------------------------------------------
// SubrangeType.java
// A subrange type.
// $Id: SubrangeType.java,v 1.7 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2009, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the types of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * An subrange type.
 *
 */
public final class SubrangeType extends TypeBase
{
  public final static String beginToken = "[";
  public final static String separatorToken = "..";
  public final static String endToken = "]";
  
  private final Term from;
  private final Term to;
  
  /****************************************************************************
   * Construct subrange type.
   * @param from the lower bound.
   * @param to the upper bound.
   * @return the type.
   ***************************************************************************/
  public static SubrangeType construct(Term from, Term to)
  {
    return new SubrangeType(from, to);
  }
  
  /****************************************************************************
   * Construct subrange type.
   * @param from the lower bound.
   * @param to the upper bound.
   ***************************************************************************/
  private SubrangeType(Term from, Term to)
  {
    this.from = from;
    this.to = to;
  }
  
  /****************************************************************************
   * Get lower bound.
   * @return the lower bound
   ***************************************************************************/
  public Term getFrom()
  {
    return from;
  }
  
  /****************************************************************************
   * Get upper bound.
   * @return the upper bound.
   ***************************************************************************/
  public Term getTo()
  {
    return to;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    from.accept(visitor);
    to.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    from.accept(visitor);
    to.accept(visitor);
  }
}

