// ----------------------------------------------------------------------------
// TypedIdentifier.java
// An identifier with a type.
// $Id: TypedIdentifier.java,v 1.5 2011/09/30 15:47:33 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.InOut.SourcePosition;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * An identifier with a type.
 * 
 */
public final class TypedIdentifier extends ASTLogicBase
  implements ValueDeclaration
{  
  private final Identifier name;
  private final Type type;
  private SourcePosition position = null;
  
  /****************************************************************************
   * Construct an identifier with a type.
   * @param name the identifier.
   * @param type its type.
   * @return the parameter.
   ***************************************************************************/
  public static TypedIdentifier construct(Identifier name, Type type)
  {
    return new TypedIdentifier(name, type);
  }
  
  /****************************************************************************
   * Construct an identifier with a type.
   * @param var the variable.
   * @param type its type.
   ***************************************************************************/
  private TypedIdentifier(Identifier name, Type type)
  {
    this.name = name;
    this.type = type;
  }
  
  /***************************************************************************
   * Get identifier.
   * @return the identifier.
   **************************************************************************/
  public Identifier getName()
  {
    return name;
  }
  
  /***************************************************************************
   * Get type.
   * @return the type.
   **************************************************************************/
  public Type getType()
  {
    return type;
  }
  
  /***************************************************************************
   * Set the position of this node in the source code.
   * @param position the source code position.
   **************************************************************************/
  public void setPosition(SourcePosition position)
  {
    this.position = position;
  }
  
  /***************************************************************************
   * Get the position of this node in the source code (may be null).
   * @return the source code position.
   **************************************************************************/
  public SourcePosition getPosition()
  {
    return position;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
    name.accept(visitor);
    type.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    name.accept(visitor);
    type.accept(visitor);
  }
}
