// ----------------------------------------------------------------------------
// ValueDeclarationInitialized.java
// The declaration of a value with initialization
// $Id: ValueDeclarationInitialized.java,v 1.3 2011/09/30 15:47:32 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Logic;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * 
 * The declaration of a value with initialization.
 *
 */
public final class ValueDeclarationInitialized extends ValueDeclarationClass
  implements ValueDefinition
{
  private final Term value;
  
  /****************************************************************************
   * Create new declaration of a constant denoting a value.
   * @param name the name of the constant.
   * @param type its type
   * @param value its value
   * @return the declaration
   ***************************************************************************/
  public static ValueDeclarationInitialized construct(Identifier name, 
      Type type, Term value)
  {
    return new ValueDeclarationInitialized(name, type, value);
  }
    
  /****************************************************************************
   * Create new declaration of a constant denoting a value.
   * @param name the name of the constant.
   * @param type its type 
   * @param value its value 
   * @return the declaration
   ***************************************************************************/
  private ValueDeclarationInitialized(Identifier name, Type type, Term value)
  {
    super(name, type);
    this.value = value;
  }

  /***************************************************************************
   * Get value of the declaration.
   * @return the value.
   **************************************************************************/
  public Term getValue()
  {
    return value;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    value.accept(visitor);
  }
}

