// ----------------------------------------------------------------------------
// LeftValue.java
// The left value of an assignment.
// $Id: LeftValue.java,v 1.4 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import java.util.*;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * The left value of an assignment.
 */
public final class LeftValue extends ASTProgramBase
{
  public final Identifier name;
  public final Selector[] selectors;
  
  /***************************************************************************
   * Construct the left value of an assignment.
   * @param name the base name of the left value.
   * @param selectors the appended selectors.
   * @return the left value.
   **************************************************************************/
  public static LeftValue construct(Identifier name, Vector<Selector> selectors)
  {
    return new LeftValue(name, selectors.toArray(new Selector[selectors.size()]));
  }
  
  /***************************************************************************
   * Construct the left value of an assignment.
   * @param name the base name of the left value.
   * @param selectors the appended selectors.
   * @return the left value.
   **************************************************************************/
  public static LeftValue construct(Identifier name, Selector[] selectors)
  {
    return new LeftValue(name, selectors);
  }
  
  /***************************************************************************
   * Construct the left value of an assignment.
   * @param name the base name of the left value.
   * @param selectors the appended selectors.
   **************************************************************************/
  private LeftValue(Identifier name, Selector[] selectors)
  {
    this.name = name;
    this.selectors = selectors;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    name.accept(visitor);
    accept(selectors, visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    name.accept(visitor);
    accept(selectors, visitor);
  }
}
