// ----------------------------------------------------------------------------
// NewArrayExpression.java
// Allocation of a new array.
// $Id: NewArrayExpression.java,v 1.4 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * Allocation of a new array.
 */
public final class NewArrayExpression extends ValueExpressionBase
{
  public static final int precedence = 13;
  
  public final TypeExpression type;
  public final ValueExpression length;
  
  /****************************************************************************
   * Allocation of a new object.
   * @param type the type of the object.
   * @param length the length of the array.
   * @return the expression.
   ***************************************************************************/
  public static NewArrayExpression construct
    (TypeExpression type, ValueExpression length)
  {
    return new NewArrayExpression(type, length);
  }
  
  /****************************************************************************
   * Allocation of a new object.
   * @param type the type of the object.
   * @param length the length of the array.
   ***************************************************************************/
  protected NewArrayExpression(TypeExpression type, ValueExpression length)
  {
    this.type = type;
    this.length = length;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    type.accept(visitor);
    length.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    type.accept(visitor);
    length.accept(visitor);
  }
  
  /***************************************************************************
   * Get precedence value of the expression operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
}
