// ----------------------------------------------------------------------------
// ParamDeclarationBase.java
// The base class for a declaration with parameters.
// $Id: ParamDeclarationBase.java,v 1.3 2011/09/30 15:47:36 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * The base class for a declaration with parameters.
 */
public abstract class ParamDeclarationBase 
extends TopDeclarationBase implements ParamDeclaration
{
  public final Parameter[] params;
  public final Statement   body;
  
  /****************************************************************************
   * Construction of a constructor declaration.
   * @param name the name of the constructor
   * @param params the constructor parameters
   * @param body the constructor body
   ***************************************************************************/
  public ParamDeclarationBase(Identifier name,
    Parameter[] params, Statement body)
  {
    super(name);
    this.params = params;
    this.body = body;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    super.accept(visitor);
    accept(params, visitor);
    body.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    accept(params, visitor);
    body.accept(visitor);
  }
  
  /****************************************************************************
   * Get parameters of declaration.
   * @return the sequence of parameters.
   ***************************************************************************/
  public Parameter[] getParams()
  {
    return params;
  }
  
  /****************************************************************************
   * Get body of declaration.
   * @return the declaration body.
   ***************************************************************************/
  public Statement getBody()
  {
    return body;
  }
}


