// ----------------------------------------------------------------------------
// Parameter.java
// A parameter i.e. a typed identifier
// $Id: Parameter.java,v 1.4 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;

/**
 * A parameter i.e. a typed identifier.
 */
public final class Parameter extends VariableDeclarationBase
{  
  /****************************************************************************
   * Construct a parameter.
   * @param type the type of the parameter.
   * @param name its name.
   * @return the parameter.
   ***************************************************************************/
  public static Parameter construct(TypeExpression type, Identifier name)
  {
    return new Parameter(type, name);
  }
  
  /****************************************************************************
   * Construct a parameter.
   * @param type the type of the parameter.
   * @param name its name.
   ***************************************************************************/
  private Parameter(TypeExpression type, Identifier name)
  {
    super(name, type);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    boolean done = visitor.visit(this);
    if (done) return;
    super.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
}
