// ----------------------------------------------------------------------------
// StatementBase.java
// The base class for statements.
// $Id: StatementBase.java,v 1.9 2011/09/30 15:47:36 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Syntax.*;
import fmrisc.ProgramExplorer.Judgements.*;

/**
 * 
 * The base class for statements.
 * 
 */
public abstract class StatementBase extends ASTSourceBase implements Statement
{
  private ProgramAnnotation annotation;
  private StatJudgement judgement;
  
  /***************************************************************************
   * Set the annotation for this item.
   * @param annotation the annotation.
   **************************************************************************/
  public void setAnnotation(ProgramAnnotation annotation)
  {
    this.annotation = annotation;
  }
  
  /***************************************************************************
   * Get the annotation for this item.
   * @return the annotation (null, if there is none).
   **************************************************************************/
  public ProgramAnnotation getAnnotation()
  {
    return annotation;
  }
  
  /****************************************************************************
   * Set command judgement.
   * @param judgement the judgement.
   ***************************************************************************/
  public void setJudgement(StatJudgement judgement)
  {
    this.judgement = judgement;
  }
  
  /****************************************************************************
   * Get command judgement.
   * @return the judgement
   ***************************************************************************/
  public StatJudgement getJudgement()
  {
    return judgement;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    super.accept(visitor);
    if (annotation != null) annotation.accept(visitor);
    if (judgement != null) judgement.accept(visitor);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit to the children of this node.
   * @param visitor the visitor who is accepted by the children of this node.
   **************************************************************************/
  public void acceptChildren(ASTVisitor visitor)
  {
    super.acceptChildren(visitor);
    if (annotation != null) annotation.accept(visitor);
    if (judgement != null) judgement.accept(visitor);
  }
}
