// ----------------------------------------------------------------------------
// StringLiteral.java
// A string literal.
// $Id: StringLiteral.java,v 1.5 2017/07/04 08:04:37 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProgramExplorer.Syntax.Program;

import fmrisc.ProgramExplorer.Semantics.ClassSymbol;
import fmrisc.ProgramExplorer.Syntax.*;

/**
 * A string literal.
 */
public final class StringLiteral extends ValueExpressionBase
{
  public static final int precedence = 16;
  
  public final String value;
  private ClassSymbol stringClass = null;
  
  /****************************************************************************
   * Construction of a string literal.
   * @param value the sequence of characters denoting the literal.
   * @return the literal.
   ***************************************************************************/
  public static StringLiteral construct(String value)
  {
    return new StringLiteral(value);
  }
  
  /****************************************************************************
   * Construction of a string literal.
   * @param value the sequence of characters denoting the literal.
   ***************************************************************************/
  protected StringLiteral(String value)
  {
    this.value = value;
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitorOld visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   **************************************************************************/
  public void accept(ASTVisitor visitor)
  {
    visitor.visit(this);
  }
  
  /***************************************************************************
   * Get precedence value of the expression operator 
   * (higher precedence value means higher binding power)
   **************************************************************************/
  public int getPrecedence()
  {
    return precedence;
  }
  
  /***************************************************************************
   * Set the string class.
   * @param stringClass the string class.
   **************************************************************************/
  public void setStringClass(ClassSymbol stringClass)
  {
    this.stringClass = stringClass;
  }
  
  /***************************************************************************
   * Get the string class.
   * @return the string class (may be null, if none).
   **************************************************************************/
  public ClassSymbol getStringClass()
  {
    return stringClass;
  }
}


