// ----------------------------------------------------------------------------
// Proved.java
// The "proved" command.
// $Id: Proved.java,v 1.9 2010/03/03 14:01:14 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.PrintWriter;
import org.w3c.dom.*;

import fmrisc.ProofNavigator.*;
import fmrisc.ProofNavigator.Proving.*;

/**
 * 
 * Pseudo-command "proved": indicate the termination of a proof branch.
 * 
 */
public final class Proved extends ProofCommandBase
{
  public final static String Name = "proved";
  private String reason; // explanation why state is proved
  
  /***************************************************************************
   * create a "proved" command.
   * @param reason explanation why state is proved
   **************************************************************************/
  public Proved(String reason)
  {
    super(Name);
    this.reason = reason;
  }
  
  /***************************************************************************
   * get reason why state has been proved
   * @return explanation why state is proved
   **************************************************************************/
  public String getReason()
  {
    return reason;
  }
  
  /****************************************************************************
   * process command "proved".
   * @return null.
   ***************************************************************************/
  protected ProofState[] processCore(ProofState state)
  {
    return null;
  }
  
  /***************************************************************************
   * Set external representation of command in a DOM node.
   * @param node the node in which the representation is to be set
   *        (by default none, to be overridden by command class)
   **************************************************************************/
  protected void setRepresentation(Node node)
  {
    node.appendChild(newTextNode(reason));
  }
  
  /***************************************************************************
   * Convert DOM node to command.
   * @param node the DOM representation of the command.
   * @return the command (null, if an error occured).
   **************************************************************************/
  public static Command newCommand(Node node)
  {
    NodeList children = node.getChildNodes();
    if (children.getLength() != 1) 
    {
      State.setError("child expected: " + node.toString());
      return null;
    }
    String reason = toString(children.item(0));
    if (reason == null) return null;
    return new Proved(reason);
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
    out.print(" (");
    out.print(reason);
    out.print(")");
  }
}
