// ----------------------------------------------------------------------------
// Read.java
// The "read" command.
// $Id: Read.java,v 1.5 2006/03/23 17:44:07 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.PrintWriter;

import fmrisc.ProofNavigator.*;

/**
 * 
 * Command "read": read command from denoted file.
 *
 */
public final class Read extends CommandBase
{
  public final static String Name = "read";
  private String path; // the name of the file to be read
  
  /***************************************************************************
   * Create a "read" command.
   * @param path the path of the file to be read.
   **************************************************************************/
  public Read(String path)
  {
    super(Name);
    this.path = path;
  }
  
  /****************************************************************************
   * Process command "read": only executed in declaration mode.
   * Switch to reading commands from denoted file
   ***************************************************************************/
  public void process()
  {
    if (State.getProof() != null)
    {
      State.setError("Command \"read\" must be not executed in proof mode.");
      return;
    }
    Main.open(path);
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
    out.print(" \"");
    out.print(path);
    out.print("\"");
  }
}
