// ----------------------------------------------------------------------------
// StateC.java
// The "state" command.
// $Id: StateC.java,v 1.7 2008/10/13 09:32:00 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Commands;

import java.io.PrintWriter;

import fmrisc.ProofNavigator.*;
import fmrisc.ProofNavigator.Proving.*;
import fmrisc.ProofNavigator.Semantics.*;
import fmrisc.ProofNavigator.Syntax.*;

/**
 * 
 * Command "state": print denoted state.
 *
 */
public final class StateC extends CommandBase
{
  public final static String Name = "state";
  
  private String label;      // state label
  private Identifier ident;  // formula identifier
  
  /***************************************************************************
   * Create a "state" command
   * @param label label of denoted state (or null, if current state)
   * @param ident formula with a proof that has the denoted state 
   *        (null, if current proof is taken)
   **************************************************************************/
  public StateC(String label, Identifier ident)
  {
    super(Name);
    this.label = label;
    this.ident = ident;
  }
  
  /****************************************************************************
   * Process command "state": show denoted state.
   ***************************************************************************/
  public void process()
  { 
    Proof proof = State.getProof();
    if (ident == null)
    {
      if (proof == null)
      {
        error("command only allowed in proof mode (use 'state <label> <formula>')");
        return;
      }
    }
    else
    {
      FormulaSymbol symbol = State.getEnvironment().getFormulaSymbol(ident);
      if (symbol == null)
      {
        error("there is no formula named " + ident.toString());
        return;
      }
      if (symbol.isAxiom())
      {
        error("formula " + ident.toString() + " is an axiom");
        return;
      }
      proof = symbol.getProof();
      if (proof == null)
      {
        error("formula " + ident.toString() + " does not have a proof");
        return;
      }
    }
    ProofState state = proof.getCurrent();
    if (label != null)
    {
      state = proof.getState(label);
      if (state == null)
      {
        error("there is no state with label " + label);
        return;
      }
    }
    PrintWriter out = State.getSystemOut();
    state.print(out);
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printCore(PrintWriter out)
  {
    out.print(getName());
    if (label != null)
    {
      out.print(" ");
      out.print(label);
    }
    if (ident != null)
    {
      out.print(" ");
      ident.print(out);
    }
  }
}