// ----------------------------------------------------------------------------
// Question.java
// A window for answering a question
// $Id: Question.java,v 1.4 2017/07/04 08:04:35 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2008, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.SWT;

import org.eclipse.swt.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;

/**
 * 
 * A window for selecting multiple directories.
 *
 */
public final class Question extends Dialog
{ 
  // the question form
  private QuestionForm form;
  
  // the information
  private String question;
  private String answer;

  // the close button
  private Button okay;
  private boolean done = false;
  
  /***************************************************************************
   * Construct question with denoted shell
   * @param parent the shell
   * @param question the text of the question
   **************************************************************************/
  public Question(Shell parent, String question) 
  {
    super(parent, SWT.APPLICATION_MODAL); 
    this.question = question;
  }

  /****************************************************************************
   * Pose question.
   * @return the answer string.
   ***************************************************************************/
  public Object open()
  {
    // setup dialog
    Shell parent = getParent();
    Shell shell = new Shell(parent, SWT.TITLE | SWT.BORDER);
    shell.setText("Question");
    shell.setSize(500, 150);

    // avoid closing the dialog by closing the window
    shell.addShellListener(new ShellAdapter() { 
      public void shellClosed(ShellEvent event)
      {
        if (!done) event.doit = false;
      }
    });

    // setup the dialog content
    form = new QuestionForm(shell);

    // manage the dialog
    shell.open();
    Display display = parent.getDisplay();
    while (!shell.isDisposed()) {
      if (!display.readAndDispatch()) display.sleep();
    }
    return answer;
  }

  /****************************************************************************
   * Set answer from external source.
   * @param answer the answer.
   ***************************************************************************/
  public void setAnswer(String answer)
  {
    final String answer0 = answer;
    form.getDisplay().syncExec(new Runnable() {
      public void run() {
        form.answerText.setText(answer0);
        int pos = answer0.indexOf("[]");
        if (pos != -1) form.answerText.setSelection(pos, pos+2);
        form.answerText.setFocus();
        form.answerText.redraw();
      }}
    );
  }
  
  private class QuestionForm extends Composite
  {
    final Text answerText;
    
    public QuestionForm(Shell shell)
    {
      super(shell, SWT.NONE);

      GridLayout shellLayout = new GridLayout();
      shell.setLayout(shellLayout);
      shellLayout.numColumns = 1;
      shellLayout.marginWidth = 0;
      shellLayout.marginHeight = 0;

      // set own layout
      setLayoutData(new GridData(GridData.FILL_BOTH));
      GridLayout topLayout =  new GridLayout();
      setLayout(topLayout);
      topLayout.numColumns = 1;
      topLayout.verticalSpacing = 10;
      topLayout.marginTop = 3;
      topLayout.marginHeight = 3;
      topLayout.marginWidth = 5;
          
      // ---------------------------------------------------------------------- 
      // question and answer
      // ----------------------------------------------------------------------
      
      Group answerGroup = new Group(this, SWT.BORDER);
      answerGroup.setText(question);
      GridLayout answerLayout = new GridLayout();
      answerGroup.setLayout(answerLayout);
      answerLayout.numColumns = 1;

      GridData answerData = new GridData(GridData.FILL_HORIZONTAL);
      answerGroup.setLayoutData(answerData);

      answerText = new Text(answerGroup, SWT.SINGLE | SWT.BORDER | 
          SWT.H_SCROLL | SWT.V_SCROLL);
      answerText.setText("");
      answerText.setLayoutData(new GridData(GridData.HORIZONTAL_ALIGN_FILL 
          | GridData.GRAB_HORIZONTAL));

      // ----------------------------------------------------------------------- 
      // button for confirming answer
      // -----------------------------------------------------------------------

      Group done = new Group(this, SWT.BORDER);
      done.setLayout(new RowLayout());
      okay = new Button(done, SWT.PUSH);
      okay.setText("Done");

      GridData doneData = new GridData();
      done.setLayoutData(doneData);
      doneData.horizontalAlignment = GridData.CENTER;

      final Shell shell0 = shell;
      okay.addSelectionListener(new SelectionAdapter() {
        public void widgetSelected(SelectionEvent e) {
          answer = answerText.getText();
          Question.this.done = true;
          shell0.close();
        }
      });
    }
  }
}


