// ----------------------------------------------------------------------------
// AST.java
// Interface to abstract syntax trees.
// $Id: AST.java,v 1.9 2008/10/13 09:32:01 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Syntax;
import java.io.*;

/**
 * Interface to abstract syntax trees.
 */
public interface AST
{
  /***************************************************************************
   * Accept visitor for a visit.
   * @param visitor the visitor who is accepted by this node.
   * @return the result of the visit (may be null).
   **************************************************************************/
  public AST accept(ASTVisitor visitor);
  
  /***************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * If ASTUtil.setParentheses(true) is called, parentheses are printed around
   * the whole expression and its subexpressions.
   * @param out the stream on which the text is written
   **************************************************************************/
  void print(PrintWriter out);
  
  /***************************************************************************
   * Like print, but parentheses around outer expression are always printed.
   * @param out the stream on which the text is written
   **************************************************************************/
  void printParens(PrintWriter out);
  
  /***************************************************************************
   * Like print, but parentheses around outer expression are not printed.
   * @param out the stream on which the text is written
   **************************************************************************/
  void printCore(PrintWriter out);
  
  /***************************************************************************
   * returns string representation of tree
   * @return the string representation
   **************************************************************************/
  public String toString();
}
