// ----------------------------------------------------------------------------
// ASTBase.java
// Base class of abstract syntax trees.
// $Id: ASTBase.java,v 1.9 2008/10/13 09:32:01 schreine Exp $
//
// Author: Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
// Copyright (C) 2005, Research Institute for Symbolic Computation (RISC)
// Johannes Kepler University, Linz, Austria, http://www.risc.uni-linz.ac.at
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
// ----------------------------------------------------------------------------
package fmrisc.ProofNavigator.Syntax;

import java.io.*;

/**
 * Base class of abstract syntax trees.
 */
public abstract class ASTBase implements AST
{   
  /***************************************************************************
   * Prints text representation on out (without new line termination). 
   * If ASTUtil.setPrintParens(true) was called, parentheses are printed 
   * around every (sub)expression.
   * @param out the stream on which the text is written
   **************************************************************************/
  public void print(PrintWriter out)
  {
    if (ASTUtil.getPrintParens()) 
      printParens(out);
    else
      printCore(out);
  }
  
  /***************************************************************************
   * Prints text representation on out (without new line termination) 
   * with parentheses around the whole expression.
   * @param out the stream on which the text is written
   **************************************************************************/
  public void printParens(PrintWriter out)
  {
    out.print('(');
    printCore(out);
    out.print(')');
  }
  
  /***************************************************************************
   * Prints text representation of tree on out (without new line termination).
   * @param out the stream on which the text is written
   **************************************************************************/
  public abstract void printCore(PrintWriter out);
  
  /***************************************************************************
   * returns string representation of tree
   * @return the string representation
   **************************************************************************/
  public String toString()
  {
    StringWriter s = new StringWriter();
    print(new PrintWriter(s));
    return s.toString();
  }
}
